### #heighttool_datum.rb tool by TIG (c) 20091020, 
### based on Robin Hill's 'Height'***
### makes height text added to a datum value that you can change, 
### the datum [Z] is saved with the model as an attribute...
#
### v1.1 20091102 text placed in active_entities.
### v1.2 20091111 datum show on change dialog, leader changed.
### v1.3 20091112 text now linked to arrowhead Z and always updates as it moves...
### v2.0 201702++ recast in module, submenu added, observers improved.
###
#####################################################
#	was HeightTool.rb
#	by Robin Hill - original idea Copyright 2004***
#
#	The tool sets up a submenu in the Plugins|Extensions Menu, its item
# 'Height Above Datum > Add Height Above Datum' is the main tool.
#	On selecting it this will become the active tool.
#	Hovering on a point shows its height above datum (default z=0=ground plane).
#	Clicking the left mouse button adds a text label with that height
#	Selecting another tool will stop this being the active tool
# Change the tool's Datum using 'Height Above Datum > Set Datum' submenu item 
# This datum is remembered with each Model as an Attribute.
# Any preexisting labels are updated to reflect the new datum.
# Convert selected text into height labels using the submenu item
# 'Height Above Datum > Make Selected Text Dynamic'
# Moving a height label vertically changes the height shown in it.
# Height labels have an observer attached which auto-updates them.
#
#	THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#####################################################
###
module TIG

class HeightTool_Datum

	def activate #called on first activation of tool - initialize here
			@ip = Sketchup::InputPoint.new
			@iptemp = Sketchup::InputPoint.new
			@displayed = false
			@pt=nil
			@vec=[2,2,2] #set this for default length of leader to text label
			@datum=Sketchup.active_model.get_attribute("HeightTool_Datum", "z", 0.0.m)
			Sketchup.active_model.set_attribute("HeightTool_Datum", "z", @datum)
			Sketchup::set_status_text("Height Datum = "+@datum.to_l.to_s, SB_PROMPT)
	end

	def onMouseMove(flags, x, y, view)
			datum=@datum.to_l
			@iptemp.pick(view, x, y) # get a position in the model 
			if @iptemp.valid?
				changed = @iptemp != @ip
				@ip.copy!(@iptemp)
				pos = @ip.position
				@pt=pos #save position so it can be used in  onLButtonUp()
				msg = @ip.tooltip #start building tooltip message
				if msg.length > 0
					msg << " "
				end
				msg << "Height: "
				msg << (pos.z+datum).to_l.to_s
				view.tooltip = msg # show message as tooltip
				# check if display needs update
				if changed && (@ip.display? || @displayed)         
					view.invalidate #make the view update  
				end
			end
	end

	def onLButtonUp(flags, x, y, view) #called when left mouse button released
			datum=@datum.to_l
			z=(@pt.z+datum).to_l
			txt="Height: #{z}"
			model=Sketchup.active_model
			model.start_operation("HeightTool_Datum", true)
				e=model.active_entities ### v1.1
				t=e.add_text(txt, @pt, @vec)
				#lyr=nil###
				#t.layer=lyr###
				t.point=@pt###
				t.display_leader=true###
				t.leader_type=1###view based
				t.arrow_type=2###dot
				t.set_attribute("HeightTool_Datum", "text", true)
				t.add_observer(TIG::HeightTextEntityObserver.new)
			model.commit_operation
	end

	def HeightTool_Datum::set_datum()
		@datum=Sketchup.active_model.get_attribute("HeightTool_Datum","z",0.0.m)
		datum=@datum
		result=inputbox(["Datum Level 'Z': "], [datum.to_l], "Set Height Datum")
		return nil unless result
		@datum=result[0]
		model=Sketchup.active_model
		model.start_operation("HeightTool_Datum: Set Datum", true)
			###
			model.set_attribute("HeightTool_Datum", "z", @datum)
			### update any existing tags
			model.entities.grep(Sketchup::Text).each{|e|
				if e.get_attribute("HeightTool_Datum", "text", false)
					e.set_text("Height: "+(e.point.z+@datum).to_l.to_s)
				end#if
			}
			model.definitions.each{|d|
				d.entities.grep(Sketchup::Text).each{|e|
					if e.get_attribute("HeightTool_Datum", "text", false)
						e.set_text("Height: "+(e.point.z+@datum).to_l.to_s)
					end#if
				}
			}
			###
		model.commit_operation
		Sketchup.send_action("selectSelectionTool:")
		return nil
	end#def

end	#end of Class  HeightTool_Datum


#####################################################
### observer creation to keep text's height updated
class TIG::HeightTextEntityObserver < Sketchup::EntityObserver
  def onChangeEntity(t) ### we update text
    datum=Sketchup.active_model.get_attribute("HeightTool_Datum", "z", 0.0.m)
    t.set_text("Height: "+(t.point.z+datum).to_l.to_s)
  end
end#class
####################################################
### set all height-text to have observers at startup
def self.addtextheightobservers()
 model=Sketchup.active_model
 if model.get_attribute("HeightTool_Datum", "z", false)
  model.entities.grep(Sketchup::Text).each{|e|
    if e.get_attribute("HeightTool_Datum", "text", false)
      e.add_observer(TIG::HeightTextEntityObserver.new)
    end#if
  }
  model.definitions.each{|d|
   d.entities.grep(Sketchup::Text).each{|e|
    if e.get_attribute("HeightTool_Datum", "text", false)
      e.add_observer(TIG::HeightTextEntityObserver.new)
    end#if
   }
  }
 end#if
end#def
###
self.addtextheightobservers()
###
### add observers on change model etc
### AppObserver 
class TIG::HeightTextAppObserver < Sketchup::AppObserver
  def onNewModel(model)  ### from menu or app
		TIG::addtextheightobservers()
  end
  def onOpenModel(model) ### from menu
		TIG::addtextheightobservers()
  end
  def onQuit() ### best to tidy up
    begin
			Sketchup.remove_observer(@HeightTextAppObserver) if @HeightTextAppObserver
			@HeightTextAppObserver = nil
		rescue
		end
  end
end#class
###
@HeightTextAppObserver = TIG::HeightTextAppObserver.new() unless @HeightTextAppObserver
Sketchup.add_observer(@HeightTextAppObserver)
#####################################################
def self.converttexttodynamicheighttext()
  model=Sketchup.active_model
  model.start_operation("Make Selected Text Dynamic", true)
  ss=model.selection
  count=0
  ss.grep(Sketchup::Text).each{|t|
    if t.point ###nil if 'screen text'
      t.display_leader=true###
      t.leader_type=1###view based
      t.arrow_type=2###dot
      datum=Sketchup.active_model.get_attribute("HeightTool_Datum", "z", 0.0.m)
      t.set_text("Height: "+(t.point.z+datum).to_l.to_s)
      t.set_attribute("HeightTool_Datum", "text", true)
      t.add_observer(TIG::HeightTextEntityObserver.new())
      count+=1
    end#if
  }
  model.commit_operation
  puts count.to_s+" text entities converted."
end#def

#####################################################

unless @Height_Tool_loaded # Add HeightTool to the plugins menu once only
    submenu = UI.menu("Plugins").add_submenu("Height Above Datum")
		submenu.add_item(">> Add Height Above Datum") { 
	    Sketchup.active_model.select_tool(HeightTool_Datum.new())
		}
    submenu.add_item(">> Set Datum") { 
			HeightTool_Datum::set_datum()
		}
		submenu.add_item(">> Make Selected Text Dynamic") { 
	    self.converttexttodynamicheighttext()
		}
    @Height_Tool_loaded = true
end

end #module
